# Stubs for networkx.algorithms.cycles (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def cycle_basis(G, root: Optional[Any] = ...): ...
def simple_cycles(G): ...
def recursive_simple_cycles(G): ...
def find_cycle(G, source: Optional[Any] = ..., orientation: str = ...): ...
def minimum_cycle_basis(G, weight: Optional[Any] = ...): ...
