# Stubs for networkx.linalg (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.linalg.algebraicconnectivity import *
from networkx.linalg.attrmatrix import *
from networkx.linalg.graphmatrix import *
from networkx.linalg.laplacianmatrix import *
from networkx.linalg.modularitymatrix import *
from networkx.linalg.spectrum import *
