# Stubs for networkx.algorithms.approximation.clique (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def max_clique(G): ...
def clique_removal(G): ...
def large_clique_size(G): ...
