# Stubs for networkx.algorithms.centrality.closeness (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def closeness_centrality(
    G,
    u: Optional[Any] = ...,
    distance: Optional[Any] = ...,
    wf_improved: bool = ...,
    reverse: bool = ...,
): ...
