# Stubs for networkx.drawing.nx_agraph (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def from_agraph(A, create_using: Optional[Any] = ...): ...
def to_agraph(N): ...
def write_dot(G, path): ...
def read_dot(path): ...
def graphviz_layout(G, prog: str = ..., root: Optional[Any] = ..., args: str = ...): ...
def pygraphviz_layout(G, prog: str = ..., root: Optional[Any] = ..., args: str = ...): ...
def view_pygraphviz(
    G,
    edgelabel: Optional[Any] = ...,
    prog: str = ...,
    args: str = ...,
    suffix: str = ...,
    path: Optional[Any] = ...,
): ...
