from rdflib.namespace import DefinedNamespace as DefinedNamespace
from rdflib.namespace import Namespace as Namespace
from rdflib.term import URIRef as URIRef

class PROV(DefinedNamespace):
    activityOfInfluence: URIRef
    agentOfInfluence: URIRef
    contributed: URIRef
    ended: URIRef
    entityOfInfluence: URIRef
    generalizationOf: URIRef
    generatedAsDerivation: URIRef
    hadDelegate: URIRef
    hadDerivation: URIRef
    hadInfluence: URIRef
    hadRevision: URIRef
    informed: URIRef
    locationOf: URIRef
    qualifiedAssociationOf: URIRef
    qualifiedAttributionOf: URIRef
    qualifiedCommunicationOf: URIRef
    qualifiedDelegationOf: URIRef
    qualifiedDerivationOf: URIRef
    qualifiedEndOf: URIRef
    qualifiedGenerationOf: URIRef
    qualifiedInfluenceOf: URIRef
    qualifiedInvalidationOf: URIRef
    qualifiedQuotationOf: URIRef
    qualifiedSourceOf: URIRef
    qualifiedStartOf: URIRef
    qualifiedUsingActivity: URIRef
    quotedAs: URIRef
    revisedEntity: URIRef
    started: URIRef
    wasActivityOfInfluence: URIRef
    wasAssociateFor: URIRef
    wasMemberOf: URIRef
    wasPlanOf: URIRef
    wasPrimarySourceOf: URIRef
    wasRoleIn: URIRef
    wasUsedBy: URIRef
    wasUsedInDerivation: URIRef
    aq: URIRef
    category: URIRef
    component: URIRef
    constraints: URIRef
    definition: URIRef
    dm: URIRef
    editorialNote: URIRef
    editorsDefinition: URIRef
    inverse: URIRef
    n: URIRef
    order: URIRef
    qualifiedForm: URIRef
    sharesDefinitionWith: URIRef
    specializationOf: URIRef
    todo: URIRef
    unqualifiedForm: URIRef
    wasRevisionOf: URIRef
    Accept: URIRef
    Activity: URIRef
    ActivityInfluence: URIRef
    Agent: URIRef
    AgentInfluence: URIRef
    Association: URIRef
    Attribution: URIRef
    Bundle: URIRef
    Collection: URIRef
    Communication: URIRef
    Contribute: URIRef
    Contributor: URIRef
    Copyright: URIRef
    Create: URIRef
    Creator: URIRef
    Delegation: URIRef
    Derivation: URIRef
    Dictionary: URIRef
    DirectQueryService: URIRef
    EmptyDictionary: URIRef
    End: URIRef
    Entity: URIRef
    EntityInfluence: URIRef
    Generation: URIRef
    Influence: URIRef
    Insertion: URIRef
    InstantaneousEvent: URIRef
    Invalidation: URIRef
    KeyEntityPair: URIRef
    Location: URIRef
    Modify: URIRef
    Organization: URIRef
    Person: URIRef
    Plan: URIRef
    PrimarySource: URIRef
    Publish: URIRef
    Publisher: URIRef
    Quotation: URIRef
    Removal: URIRef
    Replace: URIRef
    Revision: URIRef
    RightsAssignment: URIRef
    RightsHolder: URIRef
    Role: URIRef
    ServiceDescription: URIRef
    SoftwareAgent: URIRef
    Start: URIRef
    Submit: URIRef
    Usage: URIRef
    atTime: URIRef
    endedAtTime: URIRef
    generatedAtTime: URIRef
    invalidatedAtTime: URIRef
    provenanceUriTemplate: URIRef
    removedKey: URIRef
    startedAtTime: URIRef
    value: URIRef
    pairEntity: URIRef
    pairKey: URIRef
    EmptyCollection: URIRef
    actedOnBehalfOf: URIRef
    activity: URIRef
    agent: URIRef
    alternateOf: URIRef
    asInBundle: URIRef
    atLocation: URIRef
    derivedByInsertionFrom: URIRef
    derivedByRemovalFrom: URIRef
    describesService: URIRef
    dictionary: URIRef
    entity: URIRef
    generated: URIRef
    hadActivity: URIRef
    hadDictionaryMember: URIRef
    hadGeneration: URIRef
    hadMember: URIRef
    hadPlan: URIRef
    hadPrimarySource: URIRef
    hadRole: URIRef
    hadUsage: URIRef
    has_anchor: URIRef
    has_provenance: URIRef
    has_query_service: URIRef
    influenced: URIRef
    influencer: URIRef
    insertedKeyEntityPair: URIRef
    invalidated: URIRef
    mentionOf: URIRef
    pingback: URIRef
    qualifiedAssociation: URIRef
    qualifiedAttribution: URIRef
    qualifiedCommunication: URIRef
    qualifiedDelegation: URIRef
    qualifiedDerivation: URIRef
    qualifiedEnd: URIRef
    qualifiedGeneration: URIRef
    qualifiedInfluence: URIRef
    qualifiedInsertion: URIRef
    qualifiedInvalidation: URIRef
    qualifiedPrimarySource: URIRef
    qualifiedQuotation: URIRef
    qualifiedRemoval: URIRef
    qualifiedRevision: URIRef
    qualifiedStart: URIRef
    qualifiedUsage: URIRef
    used: URIRef
    wasAssociatedWith: URIRef
    wasAttributedTo: URIRef
    wasDerivedFrom: URIRef
    wasEndedBy: URIRef
    wasGeneratedBy: URIRef
    wasInfluencedBy: URIRef
    wasInformedBy: URIRef
    wasInvalidatedBy: URIRef
    wasQuotedFrom: URIRef
    wasStartedBy: URIRef
