# Stubs for networkx.readwrite.nx_yaml (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def write_yaml(G_to_be_yaml, path_for_yaml_output, **kwds): ...
def read_yaml(path): ...
