# Stubs for networkx.algorithms.distance_measures (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def extrema_bounding(G, compute: str = ...): ...
def eccentricity(G, v: Optional[Any] = ..., sp: Optional[Any] = ...): ...
def diameter(G, e: Optional[Any] = ..., usebounds: bool = ...): ...
def periphery(G, e: Optional[Any] = ..., usebounds: bool = ...): ...
def radius(G, e: Optional[Any] = ..., usebounds: bool = ...): ...
def center(G, e: Optional[Any] = ..., usebounds: bool = ...): ...
