# Stubs for networkx.classes.multidigraph (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

from networkx.classes.digraph import DiGraph
from networkx.classes.multigraph import MultiGraph

class MultiDiGraph(MultiGraph, DiGraph):
    edge_key_dict_factory: Any = ...
    def __init__(self, incoming_graph_data: Optional[Any] = ..., **attr) -> None: ...
    @property
    def adj(self): ...
    @property
    def succ(self): ...
    @property
    def pred(self): ...
    def add_edge(self, u_for_edge, v_for_edge, key: Optional[Any] = ..., **attr): ...
    def remove_edge(self, u, v, key: Optional[Any] = ...): ...
    @property
    def edges(self): ...
    out_edges: Any = ...
    @property
    def in_edges(self): ...
    @property
    def degree(self): ...
    @property
    def in_degree(self): ...
    @property
    def out_degree(self): ...
    def is_multigraph(self): ...
    def is_directed(self): ...
    def fresh_copy(self): ...
    def copy(self, as_view: bool = ...): ...
    def to_undirected(self, reciprocal: bool = ..., as_view: bool = ...): ...
    def subgraph(self, nodes): ...
    def reverse(self, copy: bool = ...): ...
