# Stubs for networkx.algorithms.shortest_paths.generic (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def has_path(G, source, target): ...
def shortest_path(
    G,
    source: Optional[Any] = ...,
    target: Optional[Any] = ...,
    weight: Optional[Any] = ...,
): ...
def shortest_path_length(
    G,
    source: Optional[Any] = ...,
    target: Optional[Any] = ...,
    weight: Optional[Any] = ...,
): ...
def average_shortest_path_length(G, weight: Optional[Any] = ...): ...
def all_shortest_paths(G, source, target, weight: Optional[Any] = ...): ...
