# Stubs for networkx.algorithms.bipartite.basic (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def color(G): ...
def is_bipartite(G): ...
def is_bipartite_node_set(G, nodes): ...
def sets(G, top_nodes: Optional[Any] = ...): ...
def density(B, nodes): ...
def degrees(B, nodes, weight: Optional[Any] = ...): ...
