# Stubs for networkx.algorithms.graphical (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def is_graphical(sequence, method: str = ...): ...
def is_valid_degree_sequence_havel_hakimi(deg_sequence): ...
def is_valid_degree_sequence_erdos_gallai(deg_sequence): ...
def is_multigraphical(sequence): ...
def is_pseudographical(sequence): ...
def is_digraphical(in_sequence, out_sequence): ...
