# Stubs for networkx.algorithms.centrality.eigenvector (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def eigenvector_centrality(
    G,
    max_iter: int = ...,
    tol: float = ...,
    nstart: Optional[Any] = ...,
    weight: Optional[Any] = ...,
): ...
def eigenvector_centrality_numpy(
    G, weight: Optional[Any] = ..., max_iter: int = ..., tol: int = ...
): ...
