# Stubs for networkx.generators.directed (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def gn_graph(
    n,
    kernel: Optional[Any] = ...,
    create_using: Optional[Any] = ...,
    seed: Optional[Any] = ...,
): ...
def gnr_graph(n, p, create_using: Optional[Any] = ..., seed: Optional[Any] = ...): ...
def gnc_graph(n, create_using: Optional[Any] = ..., seed: Optional[Any] = ...): ...
def scale_free_graph(
    n,
    alpha: float = ...,
    beta: float = ...,
    gamma: float = ...,
    delta_in: float = ...,
    delta_out: int = ...,
    create_using: Optional[Any] = ...,
    seed: Optional[Any] = ...,
): ...
def random_k_out_graph(n, k, alpha, self_loops: bool = ..., seed: Optional[Any] = ...): ...
