# Stubs for networkx.utils.misc (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

basestring = str
unicode = str

def is_string_like(obj): ...
def iterable(obj): ...
def flatten(obj, result: Optional[Any] = ...): ...
def is_list_of_ints(intlist): ...

PY2: Any

def make_str(x): ...
def generate_unique_node(): ...
def default_opener(filename): ...
def dict_to_numpy_array(d, mapping: Optional[Any] = ...): ...
def dict_to_numpy_array2(d, mapping: Optional[Any] = ...): ...
def dict_to_numpy_array1(d, mapping: Optional[Any] = ...): ...
def is_iterator(obj): ...
def arbitrary_element(iterable): ...
def consume(iterator): ...
def pairwise(iterable, cyclic: bool = ...): ...
def groups(many_to_one): ...
def to_tuple(x): ...
def create_random_state(random_state: Optional[Any] = ...): ...
