# Stubs for networkx.algorithms.richclub (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def rich_club_coefficient(G, normalized: bool = ..., Q: int = ...): ...
