# Stubs for networkx.algorithms.bipartite.spectral (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def spectral_bipartivity(G, nodes: Optional[Any] = ..., weight: str = ...): ...
