# Stubs for networkx.algorithms.operators.unary (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def complement(G): ...
def reverse(G, copy: bool = ...): ...
