# Stubs for networkx.algorithms.core (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def core_number(G): ...

find_cores = core_number

def k_core(G, k: Optional[Any] = ..., core_number: Optional[Any] = ...): ...
def k_shell(G, k: Optional[Any] = ..., core_number: Optional[Any] = ...): ...
def k_crust(G, k: Optional[Any] = ..., core_number: Optional[Any] = ...): ...
def k_corona(G, k, core_number: Optional[Any] = ...): ...
