# Stubs for networkx.readwrite.gpickle (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def write_gpickle(G, path, protocol: Any = ...): ...
def read_gpickle(path): ...
