# Stubs for networkx.generators.social (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def karate_club_graph(): ...
def davis_southern_women_graph(): ...
def florentine_families_graph(): ...
