from rdflib.namespace import DefinedNamespace as DefinedNamespace
from rdflib.namespace import Namespace as Namespace
from rdflib.term import URIRef as URIRef

class TIME(DefinedNamespace):
    generalDay: URIRef
    generalMonth: URIRef
    generalYear: URIRef
    DateTimeDescription: URIRef
    DateTimeInterval: URIRef
    DayOfWeek: URIRef
    Duration: URIRef
    DurationDescription: URIRef
    GeneralDateTimeDescription: URIRef
    GeneralDurationDescription: URIRef
    Instant: URIRef
    Interval: URIRef
    MonthOfYear: URIRef
    ProperInterval: URIRef
    TRS: URIRef
    TemporalDuration: URIRef
    TemporalEntity: URIRef
    TemporalPosition: URIRef
    TemporalUnit: URIRef
    TimePosition: URIRef
    TimeZone: URIRef
    day: URIRef
    dayOfYear: URIRef
    days: URIRef
    hasXSDDuration: URIRef
    hour: URIRef
    hours: URIRef
    inXSDDate: URIRef
    inXSDDateTimeStamp: URIRef
    inXSDgYear: URIRef
    inXSDgYearMonth: URIRef
    minute: URIRef
    minutes: URIRef
    month: URIRef
    months: URIRef
    nominalPosition: URIRef
    numericDuration: URIRef
    numericPosition: URIRef
    second: URIRef
    seconds: URIRef
    week: URIRef
    weeks: URIRef
    year: URIRef
    years: URIRef
    January: URIRef
    Year: URIRef
    inXSDDateTime: URIRef
    xsdDateTime: URIRef
    hasTRS: URIRef
    after: URIRef
    dayOfWeek: URIRef
    hasBeginning: URIRef
    hasDateTimeDescription: URIRef
    hasDuration: URIRef
    hasDurationDescription: URIRef
    hasEnd: URIRef
    hasTemporalDuration: URIRef
    hasTime: URIRef
    inDateTime: URIRef
    inTemporalPosition: URIRef
    inTimePosition: URIRef
    inside: URIRef
    intervalAfter: URIRef
    intervalBefore: URIRef
    intervalContains: URIRef
    intervalDisjoint: URIRef
    intervalDuring: URIRef
    intervalEquals: URIRef
    intervalFinishedBy: URIRef
    intervalFinishes: URIRef
    intervalIn: URIRef
    intervalMeets: URIRef
    intervalMetBy: URIRef
    intervalOverlappedBy: URIRef
    intervalOverlaps: URIRef
    intervalStartedBy: URIRef
    intervalStarts: URIRef
    monthOfYear: URIRef
    timeZone: URIRef
    unitType: URIRef
    before: URIRef
    Friday: URIRef
    Monday: URIRef
    Saturday: URIRef
    Sunday: URIRef
    Thursday: URIRef
    Tuesday: URIRef
    Wednesday: URIRef
    unitDay: URIRef
    unitHour: URIRef
    unitMinute: URIRef
    unitMonth: URIRef
    unitSecond: URIRef
    unitWeek: URIRef
    unitYear: URIRef
