# Stubs for networkx.algorithms.bipartite.matrix (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def biadjacency_matrix(
    G,
    row_order,
    column_order: Optional[Any] = ...,
    dtype: Optional[Any] = ...,
    weight: str = ...,
    format: str = ...,
): ...
def from_biadjacency_matrix(A, create_using: Optional[Any] = ..., edge_attribute: str = ...): ...
