# Stubs for networkx.release (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

basedir: Any

def write_versionfile(): ...
def get_revision(): ...
def get_info(dynamic: bool = ...): ...

name: str
major: str
minor: str
dev: bool
description: str
long_description: str
license: str
authors: Any
maintainer: str
maintainer_email: str
url: str
download_url: str
platforms: Any
keywords: Any
classifiers: Any
date: Any
date_info: Any
version: Any
version_info: Any
vcs_info: Any
