# Stubs for networkx.readwrite.json_graph.cytoscape (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def cytoscape_data(G, attrs: Optional[Any] = ...): ...
def cytoscape_graph(data, attrs: Optional[Any] = ...): ...
