# Stubs for networkx.algorithms.dominance (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def immediate_dominators(G, start): ...
def dominance_frontiers(G, start): ...
