# Stubs for networkx.algorithms.isomorphism.isomorph (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def could_be_isomorphic(G1, G2): ...

graph_could_be_isomorphic = could_be_isomorphic

def fast_could_be_isomorphic(G1, G2): ...

fast_graph_could_be_isomorphic = fast_could_be_isomorphic

def faster_could_be_isomorphic(G1, G2): ...

faster_graph_could_be_isomorphic = faster_could_be_isomorphic

def is_isomorphic(G1, G2, node_match: Optional[Any] = ..., edge_match: Optional[Any] = ...): ...
