# Stubs for networkx.algorithms.centrality.degree_alg (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def degree_centrality(G): ...
def in_degree_centrality(G): ...
def out_degree_centrality(G): ...
