# Stubs for networkx.utils.random_sequence (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def powerlaw_sequence(n, exponent: float = ...): ...
def zipf_rv(alpha, xmin: int = ..., seed: Optional[Any] = ...): ...
def cumulative_distribution(distribution): ...
def discrete_sequence(n, distribution: Optional[Any] = ..., cdistribution: Optional[Any] = ...): ...
def random_weighted_sample(mapping, k): ...
def weighted_choice(mapping): ...
