# Stubs for networkx.algorithms.community.quality (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx import NetworkXError

class NotAPartition(NetworkXError):
    def __init__(self, G, collection) -> None: ...

def performance(G, partition): ...
def coverage(G, partition): ...
def modularity(G, communities, weight: str = ...): ...
