# Stubs for networkx.readwrite.multiline_adjlist (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def generate_multiline_adjlist(G, delimiter: str = ...): ...
def write_multiline_adjlist(
    G, path, delimiter: str = ..., comments: str = ..., encoding: str = ...
): ...
def parse_multiline_adjlist(
    lines,
    comments: str = ...,
    delimiter: Optional[Any] = ...,
    create_using: Optional[Any] = ...,
    nodetype: Optional[Any] = ...,
    edgetype: Optional[Any] = ...,
): ...
def read_multiline_adjlist(
    path,
    comments: str = ...,
    delimiter: Optional[Any] = ...,
    create_using: Optional[Any] = ...,
    nodetype: Optional[Any] = ...,
    edgetype: Optional[Any] = ...,
    encoding: str = ...,
): ...
