# Stubs for graphviz._compat (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

PY2: Any
string_classes: Any
text_type = unicode

def iteritems(d): ...
def makedirs(name, mode: int = ..., exist_ok: bool = ...): ...
def stderr_write_binary(data): ...

text_type = str
