from _typeshed import Incomplete
from requests import Session

def CacheControl(
    sess: Session,
    cache: Incomplete | None = ...,
    cache_etags: bool = ...,
    serializer: Incomplete | None = ...,
    heuristic: Incomplete | None = ...,
    controller_class: Incomplete | None = ...,
    adapter_class: Incomplete | None = ...,
    cacheable_methods: Incomplete | None = ...,
) -> Session: ...
