# Stubs for networkx.generators.classic (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def full_rary_tree(r, n, create_using: Optional[Any] = ...): ...
def balanced_tree(r, h, create_using: Optional[Any] = ...): ...
def barbell_graph(m1, m2, create_using: Optional[Any] = ...): ...
def complete_graph(n, create_using: Optional[Any] = ...): ...
def circular_ladder_graph(n, create_using: Optional[Any] = ...): ...
def circulant_graph(n, offsets, create_using: Optional[Any] = ...): ...
def cycle_graph(n, create_using: Optional[Any] = ...): ...
def dorogovtsev_goltsev_mendes_graph(n, create_using: Optional[Any] = ...): ...
def empty_graph(n: int = ..., create_using: Optional[Any] = ...): ...
def ladder_graph(n, create_using: Optional[Any] = ...): ...
def lollipop_graph(m, n, create_using: Optional[Any] = ...): ...
def null_graph(create_using: Optional[Any] = ...): ...
def path_graph(n, create_using: Optional[Any] = ...): ...
def star_graph(n, create_using: Optional[Any] = ...): ...
def trivial_graph(create_using: Optional[Any] = ...): ...
def turan_graph(n, r): ...
def wheel_graph(n, create_using: Optional[Any] = ...): ...
def complete_multipartite_graph(*subset_sizes): ...
