# Stubs for networkx.generators.mycielski (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def mycielskian(G, iterations: int = ...): ...
def mycielski_graph(n): ...
