# Stubs for networkx.algorithms.swap (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def double_edge_swap(G, nswap: int = ..., max_tries: int = ...): ...
def connected_double_edge_swap(G, nswap: int = ..., _window_threshold: int = ...): ...
