# Stubs for networkx.utils.decorators (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def not_implemented_for(*graph_types): ...
def open_file(path_arg, mode: str = ...): ...
def nodes_or_number(which_args): ...
def preserve_random_state(func): ...
def random_state(random_state_index): ...
