# Stubs for networkx.algorithms.tournament (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def is_tournament(G): ...
def hamiltonian_path(G): ...
def random_tournament(n): ...
def score_sequence(G): ...
def is_reachable(G, s, t): ...
def is_strongly_connected(G): ...
