# Stubs for networkx.algorithms.community.kclique (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def k_clique_communities(G, k, cliques: Optional[Any] = ...): ...
