# Stubs for networkx.algorithms.lowest_common_ancestors (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def tree_all_pairs_lowest_common_ancestor(
    G, root: Optional[Any] = ..., pairs: Optional[Any] = ...
): ...
def lowest_common_ancestor(G, node1, node2, default: Optional[Any] = ...): ...
def all_pairs_lowest_common_ancestor(G, pairs: Optional[Any] = ...): ...
