# Stubs for networkx.algorithms.community.label_propagation (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def asyn_lpa_communities(G, weight: Optional[Any] = ...): ...
def label_propagation_communities(G): ...
