# Stubs for networkx.readwrite.gml (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

long = int
unicode = str
unichr = chr

def read_gml(path, label: str = ..., destringizer: Optional[Any] = ...): ...
def parse_gml(lines, label: str = ..., destringizer: Optional[Any] = ...): ...
def generate_gml(G, stringizer: Optional[Any] = ...): ...
def write_gml(G, path, stringizer: Optional[Any] = ...): ...
