# Stubs for networkx.algorithms.bridges (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def bridges(G, root: Optional[Any] = ...): ...
def has_bridges(G, root: Optional[Any] = ...): ...
def local_bridges(G, with_span: bool = ..., weight: Optional[Any] = ...): ...
