# Stubs for networkx.algorithms.components.biconnected (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def is_biconnected(G): ...
def biconnected_component_edges(G): ...
def biconnected_components(G): ...
def biconnected_component_subgraphs(G, copy: bool = ...): ...
def articulation_points(G): ...
