# Stubs for networkx.version (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

version: str
date: str
dev: bool
version_info: Any
date_info: Any
vcs_info: Any
