# Stubs for networkx.algorithms.dominating (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def dominating_set(G, start_with: Optional[Any] = ...): ...
def is_dominating_set(G, nbunch): ...
