# Stubs for networkx.algorithms.centrality.betweenness_subset (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def betweenness_centrality_subset(
    G, sources, targets, normalized: bool = ..., weight: Optional[Any] = ...
): ...
def edge_betweenness_centrality_subset(
    G, sources, targets, normalized: bool = ..., weight: Optional[Any] = ...
): ...
def betweenness_centrality_source(
    G, normalized: bool = ..., weight: Optional[Any] = ..., sources: Optional[Any] = ...
): ...
