# Stubs for networkx.algorithms.centrality.subgraph_alg (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.utils import *

def subgraph_centrality_exp(G): ...
def subgraph_centrality(G): ...
def communicability_betweenness_centrality(G, normalized: bool = ...): ...
def estrada_index(G): ...
