# Stubs for networkx.readwrite.nx_shp (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def read_shp(path, simplify: bool = ..., geom_attrs: bool = ..., strict: bool = ...): ...
def write_shp(G, outdir): ...
