# Stubs for networkx.generators.nonisomorphic_trees (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def nonisomorphic_trees(order, create: str = ...): ...
def number_of_nonisomorphic_trees(order): ...
