# Stubs for networkx.algorithms.operators.binary (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def union(G, H, rename: Any = ..., name: Optional[Any] = ...): ...
def disjoint_union(G, H): ...
def intersection(G, H): ...
def difference(G, H): ...
def symmetric_difference(G, H): ...
def compose(G, H): ...
