# Stubs for networkx.algorithms.flow.utils (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class CurrentEdge:
    def __init__(self, edges) -> None: ...
    def get(self): ...
    def move_to_next(self): ...

class Level:
    active: Any = ...
    inactive: Any = ...
    def __init__(self) -> None: ...

class GlobalRelabelThreshold:
    def __init__(self, n, m, freq) -> None: ...
    def add_work(self, work): ...
    def is_reached(self): ...
    def clear_work(self): ...

def build_residual_network(G, capacity): ...
def detect_unboundedness(R, s, t): ...
def build_flow_dict(G, R): ...
