# Stubs for networkx.drawing.nx_pydot (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

basestring = str
unicode = str

def write_dot(G, path): ...
def read_dot(path): ...
def from_pydot(P): ...
def to_pydot(N): ...
def graphviz_layout(G, prog: str = ..., root: Optional[Any] = ..., **kwds): ...
def pydot_layout(G, prog: str = ..., root: Optional[Any] = ..., **kwds): ...
