# Stubs for networkx.algorithms.flow.capacityscaling (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def capacity_scaling(
    G, demand: str = ..., capacity: str = ..., weight: str = ..., heap: Any = ...
): ...
