# Stubs for graphviz.lang (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def quote(identifier, html: Any = ..., valid_id: Any = ..., dot_keywords: Any = ...): ...
def quote_edge(identifier): ...
def a_list(
    label: Optional[Any] = ...,
    kwargs: Optional[Any] = ...,
    attributes: Optional[Any] = ...,
): ...
def attr_list(
    label: Optional[Any] = ...,
    kwargs: Optional[Any] = ...,
    attributes: Optional[Any] = ...,
): ...

class NoHtml: ...
