# Stubs for graphviz.tools (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def attach(object, name): ...
def mkdirs(filename, mode: int = ...): ...
def mapping_items(mapping, _iteritems: Any = ...): ...
