# Stubs for networkx.algorithms.tree.coding (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# class NotATree(nx.NetworkXException): ...

def to_nested_tuple(T, root, canonical_form: bool = ...): ...
def from_nested_tuple(sequence, sensible_relabeling: bool = ...): ...
def to_prufer_sequence(T): ...
def from_prufer_sequence(sequence): ...
