# Stubs for networkx.algorithms.community.kernighan_lin (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def kernighan_lin_bisection(
    G, partition: Optional[Any] = ..., max_iter: int = ..., weight: str = ...
): ...
