# Stubs for networkx.algorithms.operators.product (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def tensor_product(G, H): ...
def cartesian_product(G, H): ...
def lexicographic_product(G, H): ...
def strong_product(G, H): ...
def power(G, k): ...
def rooted_product(G, H, root): ...
