# Stubs for networkx.algorithms.cluster (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def triangles(G, nodes: Optional[Any] = ...): ...
def average_clustering(
    G, nodes: Optional[Any] = ..., weight: Optional[Any] = ..., count_zeros: bool = ...
): ...
def clustering(G, nodes: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def transitivity(G): ...
def square_clustering(G, nodes: Optional[Any] = ...): ...
def generalized_degree(G, nodes: Optional[Any] = ...): ...
