# Stubs for networkx.algorithms.flow.mincost (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def min_cost_flow_cost(G, demand: str = ..., capacity: str = ..., weight: str = ...): ...
def min_cost_flow(G, demand: str = ..., capacity: str = ..., weight: str = ...): ...
def cost_of_flow(G, flowDict, weight: str = ...): ...
def max_flow_min_cost(G, s, t, capacity: str = ..., weight: str = ...): ...
