# Stubs for networkx.algorithms.shortest_paths.astar (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def astar_path(G, source, target, heuristic: Optional[Any] = ..., weight: str = ...): ...
def astar_path_length(G, source, target, heuristic: Optional[Any] = ..., weight: str = ...): ...
