# Stubs for networkx.utils.rcm (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def cuthill_mckee_ordering(G, heuristic: Optional[Any] = ...): ...
def reverse_cuthill_mckee_ordering(G, heuristic: Optional[Any] = ...): ...
