# Stubs for networkx.algorithms.euler (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def is_eulerian(G): ...
def eulerian_circuit(G, source: Optional[Any] = ..., keys: bool = ...): ...
