# Stubs for networkx.generators.line (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

from networkx.utils.decorators import *

def line_graph(G, create_using: Optional[Any] = ...): ...
def inverse_line_graph(G): ...
