# Stubs for networkx.algorithms.components.attracting (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def attracting_components(G): ...
def number_attracting_components(G): ...
def is_attracting_component(G): ...
def attracting_component_subgraphs(G, copy: bool = ...): ...
