# Stubs for networkx.algorithms.traversal.depth_first_search (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def dfs_edges(G, source: Optional[Any] = ..., depth_limit: Optional[Any] = ...): ...
def dfs_tree(G, source: Optional[Any] = ..., depth_limit: Optional[Any] = ...): ...
def dfs_predecessors(G, source: Optional[Any] = ..., depth_limit: Optional[Any] = ...): ...
def dfs_successors(G, source: Optional[Any] = ..., depth_limit: Optional[Any] = ...): ...
def dfs_postorder_nodes(G, source: Optional[Any] = ..., depth_limit: Optional[Any] = ...): ...
def dfs_preorder_nodes(G, source: Optional[Any] = ..., depth_limit: Optional[Any] = ...): ...
def dfs_labeled_edges(G, source: Optional[Any] = ..., depth_limit: Optional[Any] = ...): ...
