# Stubs for networkx.algorithms.approximation.clustering_coefficient (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def average_clustering(G, trials: int = ...): ...
