# Stubs for networkx.algorithms.connectivity.utils (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def build_auxiliary_node_connectivity(G): ...
def build_auxiliary_edge_connectivity(G): ...
