# Stubs for networkx.algorithms.traversal.beamsearch (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def bfs_beam_edges(G, source, value, width: Optional[Any] = ...): ...
