# Stubs for networkx.algorithms.approximation.connectivity (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def local_node_connectivity(G, source, target, cutoff: Optional[Any] = ...): ...
def node_connectivity(G, s: Optional[Any] = ..., t: Optional[Any] = ...): ...
def all_pairs_node_connectivity(G, nbunch: Optional[Any] = ..., cutoff: Optional[Any] = ...): ...
