# Stubs for networkx.readwrite.p2g (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def write_p2g(G, path, encoding: str = ...): ...
def read_p2g(path, encoding: str = ...): ...
def parse_p2g(lines): ...
