# Stubs for networkx.readwrite.json_graph.adjacency (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def adjacency_data(G, attrs: Any = ...): ...
def adjacency_graph(data, directed: bool = ..., multigraph: bool = ..., attrs: Any = ...): ...
