# Stubs for networkx.algorithms.traversal.edgedfs (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def edge_dfs(G, source: Optional[Any] = ..., orientation: str = ...): ...
