# Stubs for networkx.algorithms.community.centrality (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def girvan_newman(G, most_valuable_edge: Optional[Any] = ...): ...
