# Stubs for networkx.algorithms.connectivity.stoerwagner (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def stoer_wagner(G, weight: str = ..., heap: Any = ...): ...
