# Stubs for networkx.algorithms.dag (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def descendants(G, source): ...
def ancestors(G, source): ...
def is_directed_acyclic_graph(G): ...
def topological_sort(G): ...
def lexicographical_topological_sort(G, key: Optional[Any] = ...): ...
def is_aperiodic(G): ...
def transitive_closure(G): ...
def transitive_reduction(G): ...
def antichains(G): ...
def dag_longest_path(G, weight: str = ..., default_weight: int = ...): ...
def dag_longest_path_length(G, weight: str = ..., default_weight: int = ...): ...
def dag_to_branching(G): ...
