# Stubs for networkx.generators.joint_degree_seq (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def is_valid_joint_degree(joint_degrees): ...
def joint_degree_graph(joint_degrees, seed: Optional[Any] = ...): ...
