# Stubs for networkx.algorithms.shortest_paths.dense (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def floyd_warshall_numpy(G, nodelist: Optional[Any] = ..., weight: str = ...): ...
def floyd_warshall_predecessor_and_distance(G, weight: str = ...): ...
def floyd_warshall(G, weight: str = ...): ...
