# Stubs for networkx.algorithms.bipartite.matching (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def hopcroft_karp_matching(G, top_nodes: Optional[Any] = ...): ...
def eppstein_matching(G, top_nodes: Optional[Any] = ...): ...
def to_vertex_cover(G, matching, top_nodes: Optional[Any] = ...): ...

maximum_matching = hopcroft_karp_matching
