# Stubs for networkx.algorithms.tree.recognition (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def is_arborescence(G): ...
def is_branching(G): ...
def is_forest(G): ...
def is_tree(G): ...
