# Stubs for networkx.algorithms.isolate (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def is_isolate(G, n): ...
def isolates(G): ...
def number_of_isolates(G): ...
