# Stubs for networkx.readwrite.sparse6 (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def from_sparse6_bytes(string): ...
def to_sparse6_bytes(G, nodes: Optional[Any] = ..., header: bool = ...): ...
def read_sparse6(path): ...
def write_sparse6(G, path, nodes: Optional[Any] = ..., header: bool = ...): ...
