# Stubs for networkx.classes (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .digraph import DiGraph
from .function import *
from .graph import Graph
from .multidigraph import MultiDiGraph
from .multigraph import MultiGraph
from .ordered import *
