# Stubs for networkx.readwrite.graph6 (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def from_graph6_bytes(string): ...
def to_graph6_bytes(G, nodes: Optional[Any] = ..., header: bool = ...): ...
def read_graph6(path): ...
def write_graph6(G, path, nodes: Optional[Any] = ..., header: bool = ...): ...
