# Stubs for networkx.algorithms.traversal.breadth_first_search (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def bfs_edges(G, source, reverse: bool = ...): ...
def bfs_tree(G, source, reverse: bool = ...): ...
def bfs_predecessors(G, source): ...
def bfs_successors(G, source): ...
