# Stubs for networkx.algorithms.connectivity.kcutsets (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

from networkx.algorithms.flow import edmonds_karp

default_flow_func = edmonds_karp

def all_node_cuts(G, k: Optional[Any] = ..., flow_func: Optional[Any] = ...): ...
