# Stubs for networkx.algorithms.coloring.greedy_coloring (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def strategy_largest_first(G, colors): ...
def strategy_random_sequential(G, colors): ...
def strategy_smallest_last(G, colors): ...
def strategy_independent_set(G, colors): ...
def strategy_connected_sequential_bfs(G, colors): ...
def strategy_connected_sequential_dfs(G, colors): ...
def strategy_connected_sequential(G, colors, traversal: str = ...): ...
def strategy_saturation_largest_first(G, colors): ...
def greedy_color(G, strategy: str = ..., interchange: bool = ...): ...
