# Stubs for networkx.generators.trees (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def prefix_tree(paths): ...
def random_tree(n, seed: Optional[Any] = ...): ...
