# Stubs for networkx.linalg.spectrum (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def laplacian_spectrum(G, weight: str = ...): ...
def adjacency_spectrum(G, weight: str = ...): ...
def modularity_spectrum(G): ...
