# Stubs for networkx.algorithms.threshold (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def is_threshold_graph(G): ...
def find_threshold_graph(G, create_using: Optional[Any] = ...): ...
