# Stubs for networkx.generators.expanders (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def margulis_gabber_galil_graph(n, create_using: Optional[Any] = ...): ...
def chordal_cycle_graph(p, create_using: Optional[Any] = ...): ...
