# Stubs for networkx.readwrite.pajek (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def generate_pajek(G): ...
def write_pajek(G, path, encoding: str = ...): ...
def read_pajek(path, encoding: str = ...): ...
def parse_pajek(lines): ...
