# Stubs for networkx.algorithms.components.strongly_connected (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def strongly_connected_components(G): ...
def kosaraju_strongly_connected_components(G, source: Optional[Any] = ...): ...
def strongly_connected_components_recursive(G): ...
def strongly_connected_component_subgraphs(G, copy: bool = ...): ...
def number_strongly_connected_components(G): ...
def is_strongly_connected(G): ...
def condensation(G, scc: Optional[Any] = ...): ...
