# Stubs for networkx.algorithms.centrality.dispersion (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def dispersion(
    G,
    u: Optional[Any] = ...,
    v: Optional[Any] = ...,
    normalized: bool = ...,
    alpha: float = ...,
    b: float = ...,
    c: float = ...,
): ...
