# Stubs for networkx.algorithms.approximation.vertex_cover (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def min_weighted_vertex_cover(G, weight: Optional[Any] = ...): ...
