from _typeshed import Incomplete

logger: Incomplete
URI: Incomplete

class CacheController:
    cache: Incomplete
    cache_etags: Incomplete
    serializer: Incomplete
    cacheable_status_codes: Incomplete
    def __init__(
        self,
        cache: Incomplete | None = ...,
        cache_etags: bool = ...,
        serializer: Incomplete | None = ...,
        status_codes: Incomplete | None = ...,
    ) -> None: ...
