# Stubs for networkx.algorithms.shortest_paths (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.shortest_paths.astar import *
from networkx.algorithms.shortest_paths.dense import *
from networkx.algorithms.shortest_paths.generic import *
from networkx.algorithms.shortest_paths.unweighted import *
from networkx.algorithms.shortest_paths.weighted import *
