# Stubs for networkx.generators.lattice (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def grid_2d_graph(m, n, periodic: bool = ..., create_using: Optional[Any] = ...): ...
def grid_graph(dim, periodic: bool = ...): ...
def hypercube_graph(n): ...
def triangular_lattice_graph(
    m,
    n,
    periodic: bool = ...,
    with_positions: bool = ...,
    create_using: Optional[Any] = ...,
): ...
def hexagonal_lattice_graph(
    m,
    n,
    periodic: bool = ...,
    with_positions: bool = ...,
    create_using: Optional[Any] = ...,
): ...
