# Stubs for networkx.algorithms.operators.all (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def union_all(graphs, rename: Any = ...): ...
def disjoint_union_all(graphs): ...
def compose_all(graphs): ...
def intersection_all(graphs): ...
