# Stubs for networkx.readwrite.json_graph.jit (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def jit_graph(data, create_using: Optional[Any] = ...): ...
def jit_data(G, indent: Optional[Any] = ...): ...
