# Stubs for networkx.algorithms.distance_regular (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def is_distance_regular(G): ...
def global_parameters(b, c): ...
def intersection_array(G): ...
def is_strongly_regular(G): ...
