# Stubs for networkx.generators.stochastic (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def stochastic_graph(G, copy: bool = ..., weight: str = ...): ...
