# Stubs for networkx.generators.duplication (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def partial_duplication_graph(N, n, p, q, seed: Optional[Any] = ...): ...
def duplication_divergence_graph(n, p, seed: Optional[Any] = ...): ...
