# Stubs for networkx.algorithms.bipartite.generators (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def complete_bipartite_graph(n1, n2, create_using: Optional[Any] = ...): ...
def configuration_model(
    aseq, bseq, create_using: Optional[Any] = ..., seed: Optional[Any] = ...
): ...
def havel_hakimi_graph(aseq, bseq, create_using: Optional[Any] = ...): ...
def reverse_havel_hakimi_graph(aseq, bseq, create_using: Optional[Any] = ...): ...
def alternating_havel_hakimi_graph(aseq, bseq, create_using: Optional[Any] = ...): ...
def preferential_attachment_graph(
    aseq, p, create_using: Optional[Any] = ..., seed: Optional[Any] = ...
): ...
def random_graph(n, m, p, seed: Optional[Any] = ..., directed: bool = ...): ...
def gnmk_random_graph(n, m, k, seed: Optional[Any] = ..., directed: bool = ...): ...
