# Stubs for networkx.algorithms.approximation.dominating_set (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def min_weighted_dominating_set(G, weight: Optional[Any] = ...): ...
def min_edge_dominating_set(G): ...
