# Stubs for networkx.relabel (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def relabel_nodes(G, mapping, copy: bool = ...): ...
def convert_node_labels_to_integers(
    G, first_label: int = ..., ordering: str = ..., label_attribute: Optional[Any] = ...
): ...
