# Stubs for networkx.classes.filters (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def no_filter(*items): ...
def hide_nodes(nodes): ...
def hide_diedges(edges): ...
def hide_edges(edges): ...
def hide_multidiedges(edges): ...
def hide_multiedges(edges): ...

class show_nodes:
    nodes: Any = ...
    def __init__(self, nodes) -> None: ...
    def __call__(self, node): ...

def show_diedges(edges): ...
def show_edges(edges): ...
def show_multidiedges(edges): ...
def show_multiedges(edges): ...
