# Stubs for networkx.algorithms.covering (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def min_edge_cover(G, matching_algorithm: Optional[Any] = ...): ...
def is_edge_cover(G, cover): ...
