# Stubs for networkx.algorithms.cuts (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def cut_size(G, S, T: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def volume(G, S, weight: Optional[Any] = ...): ...
def normalized_cut_size(G, S, T: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def conductance(G, S, T: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def edge_expansion(G, S, T: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def mixing_expansion(G, S, T: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def node_expansion(G, S): ...
def boundary_expansion(G, S): ...
