# Stubs for networkx.algorithms.coloring.greedy_coloring_with_interchange (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Node:
    node_id: Any = ...
    color: int = ...
    adj_list: Any = ...
    adj_color: Any = ...
    def __init__(self, node_id, n) -> None: ...
    def assign_color(self, adj_entry, color): ...
    def clear_color(self, adj_entry, color): ...
    def iter_neighbors(self): ...
    def iter_neighbors_color(self, color): ...

class AdjEntry:
    node_id: Any = ...
    next: Any = ...
    mate: Any = ...
    col_next: Any = ...
    col_prev: Any = ...
    def __init__(self, node_id) -> None: ...

def greedy_coloring_with_interchange(original_graph, nodes): ...
