# Stubs for networkx.drawing.layout (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def random_layout(
    G, center: Optional[Any] = ..., dim: int = ..., random_state: Optional[Any] = ...
): ...
def circular_layout(G, scale: int = ..., center: Optional[Any] = ..., dim: int = ...): ...
def shell_layout(
    G,
    nlist: Optional[Any] = ...,
    scale: int = ...,
    center: Optional[Any] = ...,
    dim: int = ...,
): ...
def fruchterman_reingold_layout(
    G,
    k: Optional[Any] = ...,
    pos: Optional[Any] = ...,
    fixed: Optional[Any] = ...,
    iterations: int = ...,
    threshold: float = ...,
    weight: str = ...,
    scale: int = ...,
    center: Optional[Any] = ...,
    dim: int = ...,
    random_state: Optional[Any] = ...,
): ...

spring_layout = fruchterman_reingold_layout

def kamada_kawai_layout(
    G,
    dist: Optional[Any] = ...,
    pos: Optional[Any] = ...,
    weight: str = ...,
    scale: int = ...,
    center: Optional[Any] = ...,
    dim: int = ...,
): ...
def spectral_layout(
    G, weight: str = ..., scale: int = ..., center: Optional[Any] = ..., dim: int = ...
): ...
def rescale_layout(pos, scale: int = ...): ...
