# Stubs for networkx.readwrite.json_graph.node_link (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def node_link_data(G, attrs: Optional[Any] = ...): ...
def node_link_graph(
    data, directed: bool = ..., multigraph: bool = ..., attrs: Optional[Any] = ...
): ...
