# Stubs for networkx.algorithms.hybrid (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def kl_connected_subgraph(G, k, l, low_memory: bool = ..., same_as_graph: bool = ...): ...
def is_kl_connected(G, k, l, low_memory: bool = ...): ...
