# Stubs for networkx.algorithms.isomorphism.matchhelpers (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def categorical_node_match(attr, default): ...

categorical_edge_match: Any

def categorical_multiedge_match(attr, default): ...
def numerical_node_match(attr, default, rtol: float = ..., atol: float = ...): ...

numerical_edge_match: Any

def numerical_multiedge_match(attr, default, rtol: float = ..., atol: float = ...): ...
def generic_node_match(attr, default, op): ...

generic_edge_match: Any

def generic_multiedge_match(attr, default, op): ...
