from rdflib.namespace import DefinedNamespace as DefinedNamespace
from rdflib.namespace import Namespace as Namespace
from rdflib.term import URIRef as URIRef

class SH(DefinedNamespace):
    alternativePath: URIRef
    annotationProperty: URIRef
    annotationValue: URIRef
    annotationVarName: URIRef
    ask: URIRef
    closed: URIRef
    condition: URIRef
    conforms: URIRef
    construct: URIRef
    datatype: URIRef
    deactivated: URIRef
    declare: URIRef
    defaultValue: URIRef
    description: URIRef
    detail: URIRef
    disjoint: URIRef
    entailment: URIRef
    equals: URIRef
    expression: URIRef
    filterShape: URIRef
    flags: URIRef
    focusNode: URIRef
    group: URIRef
    hasValue: URIRef
    ignoredProperties: URIRef
    intersection: URIRef
    inversePath: URIRef
    js: URIRef
    jsFunctionName: URIRef
    jsLibrary: URIRef
    jsLibraryURL: URIRef
    labelTemplate: URIRef
    languageIn: URIRef
    lessThan: URIRef
    lessThanOrEquals: URIRef
    maxCount: URIRef
    maxExclusive: URIRef
    maxInclusive: URIRef
    maxLength: URIRef
    message: URIRef
    minCount: URIRef
    minExclusive: URIRef
    minInclusive: URIRef
    minLength: URIRef
    name: URIRef
    namespace: URIRef
    node: URIRef
    nodeKind: URIRef
    nodeValidator: URIRef
    nodes: URIRef
    object: URIRef
    oneOrMorePath: URIRef
    optional: URIRef
    order: URIRef
    parameter: URIRef
    path: URIRef
    pattern: URIRef
    predicate: URIRef
    prefix: URIRef
    prefixes: URIRef
    property: URIRef
    propertyValidator: URIRef
    qualifiedMaxCount: URIRef
    qualifiedMinCount: URIRef
    qualifiedValueShape: URIRef
    qualifiedValueShapesDisjoint: URIRef
    result: URIRef
    resultAnnotation: URIRef
    resultMessage: URIRef
    resultPath: URIRef
    resultSeverity: URIRef
    returnType: URIRef
    rule: URIRef
    select: URIRef
    severity: URIRef
    shapesGraph: URIRef
    shapesGraphWellFormed: URIRef
    sourceConstraint: URIRef
    sourceConstraintComponent: URIRef
    sourceShape: URIRef
    sparql: URIRef
    subject: URIRef
    suggestedShapesGraph: URIRef
    target: URIRef
    targetClass: URIRef
    targetNode: URIRef
    targetObjectsOf: URIRef
    targetSubjectsOf: URIRef
    union: URIRef
    uniqueLang: URIRef
    update: URIRef
    validator: URIRef
    value: URIRef
    xone: URIRef
    zeroOrMorePath: URIRef
    zeroOrOnePath: URIRef
    AbstractResult: URIRef
    ConstraintComponent: URIRef
    Function: URIRef
    JSConstraint: URIRef
    JSExecutable: URIRef
    JSFunction: URIRef
    JSLibrary: URIRef
    JSRule: URIRef
    JSTarget: URIRef
    JSTargetType: URIRef
    JSValidator: URIRef
    NodeKind: URIRef
    NodeShape: URIRef
    Parameter: URIRef
    Parameterizable: URIRef
    PrefixDeclaration: URIRef
    PropertyGroup: URIRef
    PropertyShape: URIRef
    ResultAnnotation: URIRef
    Rule: URIRef
    SPARQLAskExecutable: URIRef
    SPARQLAskValidator: URIRef
    SPARQLConstraint: URIRef
    SPARQLConstructExecutable: URIRef
    SPARQLExecutable: URIRef
    SPARQLFunction: URIRef
    SPARQLRule: URIRef
    SPARQLSelectExecutable: URIRef
    SPARQLSelectValidator: URIRef
    SPARQLTarget: URIRef
    SPARQLTargetType: URIRef
    SPARQLUpdateExecutable: URIRef
    Severity: URIRef
    Shape: URIRef
    Target: URIRef
    TargetType: URIRef
    TripleRule: URIRef
    ValidationReport: URIRef
    ValidationResult: URIRef
    Validator: URIRef
    this: URIRef
    AndConstraintComponent: URIRef
    ClassConstraintComponent: URIRef
    ClosedConstraintComponent: URIRef
    DatatypeConstraintComponent: URIRef
    DisjointConstraintComponent: URIRef
    EqualsConstraintComponent: URIRef
    ExpressionConstraintComponent: URIRef
    HasValueConstraintComponent: URIRef
    InConstraintComponent: URIRef
    JSConstraintComponent: URIRef
    LanguageInConstraintComponent: URIRef
    LessThanConstraintComponent: URIRef
    LessThanOrEqualsConstraintComponent: URIRef
    MaxCountConstraintComponent: URIRef
    MaxExclusiveConstraintComponent: URIRef
    MaxInclusiveConstraintComponent: URIRef
    MaxLengthConstraintComponent: URIRef
    MinCountConstraintComponent: URIRef
    MinExclusiveConstraintComponent: URIRef
    MinInclusiveConstraintComponent: URIRef
    MinLengthConstraintComponent: URIRef
    NodeConstraintComponent: URIRef
    NodeKindConstraintComponent: URIRef
    NotConstraintComponent: URIRef
    OrConstraintComponent: URIRef
    PatternConstraintComponent: URIRef
    PropertyConstraintComponent: URIRef
    QualifiedMaxCountConstraintComponent: URIRef
    QualifiedMinCountConstraintComponent: URIRef
    SPARQLConstraintComponent: URIRef
    UniqueLangConstraintComponent: URIRef
    XoneConstraintComponent: URIRef
    BlankNode: URIRef
    BlankNodeOrIRI: URIRef
    BlankNodeOrLiteral: URIRef
    IRI: URIRef
    IRIOrLiteral: URIRef
    Literal: URIRef
    Info: URIRef
    Violation: URIRef
    Warning: URIRef
