# Stubs for networkx.algorithms.centrality.harmonic (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def harmonic_centrality(G, nbunch: Optional[Any] = ..., distance: Optional[Any] = ...): ...
