# Stubs for networkx.algorithms.tree.mst (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def minimum_spanning_edges(
    G,
    algorithm: str = ...,
    weight: str = ...,
    keys: bool = ...,
    data: bool = ...,
    ignore_nan: bool = ...,
): ...
def maximum_spanning_edges(
    G,
    algorithm: str = ...,
    weight: str = ...,
    keys: bool = ...,
    data: bool = ...,
    ignore_nan: bool = ...,
): ...
def minimum_spanning_tree(G, weight: str = ..., algorithm: str = ..., ignore_nan: bool = ...): ...
def maximum_spanning_tree(G, weight: str = ..., algorithm: str = ..., ignore_nan: bool = ...): ...
