# Stubs for networkx.algorithms.flow.gomory_hu (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

from .edmondskarp import edmonds_karp

default_flow_func = edmonds_karp

def gomory_hu_tree(G, capacity: str = ..., flow_func: Optional[Any] = ...): ...
