# Stubs for graphviz.backend (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class ExecutableNotFound(RuntimeError):
    def __init__(self, args) -> None: ...

def render(engine, format, filepath, quiet: bool = ...): ...
def pipe(engine, format, data, quiet: bool = ...): ...
def version(): ...
def view(filepath): ...
