# Stubs for networkx.algorithms.matching (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def maximal_matching(G): ...
def is_matching(G, matching): ...
def is_maximal_matching(G, matching): ...
def max_weight_matching(G, maxcardinality: bool = ..., weight: str = ...): ...
