# Stubs for networkx.algorithms.clique (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def enumerate_all_cliques(G): ...
def find_cliques(G): ...
def find_cliques_recursive(G): ...
def make_max_clique_graph(G, create_using: Optional[Any] = ...): ...
def make_clique_bipartite(
    G,
    fpos: Optional[Any] = ...,
    create_using: Optional[Any] = ...,
    name: Optional[Any] = ...,
): ...
def graph_clique_number(G, cliques: Optional[Any] = ...): ...
def graph_number_of_cliques(G, cliques: Optional[Any] = ...): ...
def node_clique_number(G, nodes: Optional[Any] = ..., cliques: Optional[Any] = ...): ...
def number_of_cliques(G, nodes: Optional[Any] = ..., cliques: Optional[Any] = ...): ...
def cliques_containing_node(G, nodes: Optional[Any] = ..., cliques: Optional[Any] = ...): ...
