# Stubs for networkx.algorithms.bipartite.projection (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def projected_graph(B, nodes, multigraph: bool = ...): ...
def weighted_projected_graph(B, nodes, ratio: bool = ...): ...
def collaboration_weighted_projected_graph(B, nodes): ...
def overlap_weighted_projected_graph(B, nodes, jaccard: bool = ...): ...
def generic_weighted_projected_graph(B, nodes, weight_function: Optional[Any] = ...): ...
def project(B, nodes, create_using: Optional[Any] = ...): ...
