# Stubs for networkx.algorithms.coloring (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.algorithms.coloring.greedy_coloring import *

# Names in __all__ with no definition:
#   greedy_color
