# Stubs for networkx.convert (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def to_networkx_graph(data, create_using: Optional[Any] = ..., multigraph_input: bool = ...): ...
def to_dict_of_lists(G, nodelist: Optional[Any] = ...): ...
def from_dict_of_lists(d, create_using: Optional[Any] = ...): ...
def to_dict_of_dicts(G, nodelist: Optional[Any] = ..., edge_data: Optional[Any] = ...): ...
def from_dict_of_dicts(d, create_using: Optional[Any] = ..., multigraph_input: bool = ...): ...
def to_edgelist(G, nodelist: Optional[Any] = ...): ...
def from_edgelist(edgelist, create_using: Optional[Any] = ...): ...
