# Stubs for networkx.algorithms.chains (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def chain_decomposition(G, root: Optional[Any] = ...): ...
