# Stubs for networkx.linalg.algebraicconnectivity (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

class _PCGSolver:
    def __init__(self, A, M) -> None: ...
    def solve(self, B, tol): ...

class _CholeskySolver:
    def __init__(self, A) -> None: ...
    def solve(self, B, tol: Optional[Any] = ...): ...

class _LUSolver:
    def __init__(self, A) -> None: ...
    def solve(self, B, tol: Optional[Any] = ...): ...

def algebraic_connectivity(
    G, weight: str = ..., normalized: bool = ..., tol: float = ..., method: str = ...
): ...
def fiedler_vector(
    G, weight: str = ..., normalized: bool = ..., tol: float = ..., method: str = ...
): ...
def spectral_ordering(
    G, weight: str = ..., normalized: bool = ..., tol: float = ..., method: str = ...
): ...
def setup_module(module): ...
