from rdflib.namespace import DefinedNamespace as DefinedNamespace
from rdflib.namespace import Namespace as Namespace
from rdflib.term import URIRef as URIRef

class DCTERMS(DefinedNamespace):
    DCMIType: URIRef
    DDC: URIRef
    IMT: URIRef
    LCC: URIRef
    LCSH: URIRef
    MESH: URIRef
    NLM: URIRef
    TGN: URIRef
    UDC: URIRef
    abstract: URIRef
    accessRights: URIRef
    accrualMethod: URIRef
    accrualPeriodicity: URIRef
    accrualPolicy: URIRef
    alternative: URIRef
    audience: URIRef
    available: URIRef
    bibliographicCitation: URIRef
    conformsTo: URIRef
    contributor: URIRef
    coverage: URIRef
    created: URIRef
    creator: URIRef
    date: URIRef
    dateAccepted: URIRef
    dateCopyrighted: URIRef
    dateSubmitted: URIRef
    description: URIRef
    educationLevel: URIRef
    extent: URIRef
    format: URIRef
    hasFormat: URIRef
    hasPart: URIRef
    hasVersion: URIRef
    identifier: URIRef
    instructionalMethod: URIRef
    isFormatOf: URIRef
    isPartOf: URIRef
    isReferencedBy: URIRef
    isReplacedBy: URIRef
    isRequiredBy: URIRef
    isVersionOf: URIRef
    issued: URIRef
    language: URIRef
    license: URIRef
    mediator: URIRef
    medium: URIRef
    modified: URIRef
    provenance: URIRef
    publisher: URIRef
    references: URIRef
    relation: URIRef
    replaces: URIRef
    requires: URIRef
    rights: URIRef
    rightsHolder: URIRef
    source: URIRef
    spatial: URIRef
    subject: URIRef
    tableOfContents: URIRef
    temporal: URIRef
    title: URIRef
    type: URIRef
    valid: URIRef
    Agent: URIRef
    AgentClass: URIRef
    BibliographicResource: URIRef
    FileFormat: URIRef
    Frequency: URIRef
    Jurisdiction: URIRef
    LicenseDocument: URIRef
    LinguisticSystem: URIRef
    Location: URIRef
    LocationPeriodOrJurisdiction: URIRef
    MediaType: URIRef
    MediaTypeOrExtent: URIRef
    MethodOfAccrual: URIRef
    MethodOfInstruction: URIRef
    PeriodOfTime: URIRef
    PhysicalMedium: URIRef
    PhysicalResource: URIRef
    Policy: URIRef
    ProvenanceStatement: URIRef
    RightsStatement: URIRef
    SizeOrDuration: URIRef
    Standard: URIRef
    Box: URIRef
    ISO3166: URIRef
    Period: URIRef
    Point: URIRef
    RFC1766: URIRef
    RFC3066: URIRef
    RFC4646: URIRef
    RFC5646: URIRef
    URI: URIRef
    W3CDTF: URIRef
