# Stubs for networkx.generators.community (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def caveman_graph(l, k): ...
def connected_caveman_graph(l, k): ...
def relaxed_caveman_graph(l, k, p, seed: Optional[Any] = ...): ...
def random_partition_graph(sizes, p_in, p_out, seed: Optional[Any] = ..., directed: bool = ...): ...
def planted_partition_graph(l, k, p_in, p_out, seed: Optional[Any] = ..., directed: bool = ...): ...
def gaussian_random_partition_graph(
    n, s, v, p_in, p_out, directed: bool = ..., seed: Optional[Any] = ...
): ...
def ring_of_cliques(num_cliques, clique_size): ...
def windmill_graph(n, k): ...
