# Stubs for networkx.readwrite.json_graph.tree (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def tree_data(G, root, attrs: Any = ...): ...
def tree_graph(data, attrs: Any = ...): ...
