# Stubs for networkx.linalg.attrmatrix (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def attr_matrix(
    G,
    edge_attr: Optional[Any] = ...,
    node_attr: Optional[Any] = ...,
    normalized: bool = ...,
    rc_order: Optional[Any] = ...,
    dtype: Optional[Any] = ...,
    order: Optional[Any] = ...,
): ...
def attr_sparse_matrix(
    G,
    edge_attr: Optional[Any] = ...,
    node_attr: Optional[Any] = ...,
    normalized: bool = ...,
    rc_order: Optional[Any] = ...,
    dtype: Optional[Any] = ...,
): ...
