# Stubs for networkx.algorithms.structuralholes (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def effective_size(G, nodes: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def constraint(G, nodes: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def local_constraint(G, u, v, weight: Optional[Any] = ...): ...
