# Stubs for networkx.generators.random_clustered (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def random_clustered_graph(
    joint_degree_sequence, create_using: Optional[Any] = ..., seed: Optional[Any] = ...
): ...
