# Stubs for networkx.generators.small (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def make_small_graph(graph_description, create_using: Optional[Any] = ...): ...
def LCF_graph(n, shift_list, repeats, create_using: Optional[Any] = ...): ...
def bull_graph(create_using: Optional[Any] = ...): ...
def chvatal_graph(create_using: Optional[Any] = ...): ...
def cubical_graph(create_using: Optional[Any] = ...): ...
def desargues_graph(create_using: Optional[Any] = ...): ...
def diamond_graph(create_using: Optional[Any] = ...): ...
def dodecahedral_graph(create_using: Optional[Any] = ...): ...
def frucht_graph(create_using: Optional[Any] = ...): ...
def heawood_graph(create_using: Optional[Any] = ...): ...
def hoffman_singleton_graph(): ...
def house_graph(create_using: Optional[Any] = ...): ...
def house_x_graph(create_using: Optional[Any] = ...): ...
def icosahedral_graph(create_using: Optional[Any] = ...): ...
def krackhardt_kite_graph(create_using: Optional[Any] = ...): ...
def moebius_kantor_graph(create_using: Optional[Any] = ...): ...
def octahedral_graph(create_using: Optional[Any] = ...): ...
def pappus_graph(): ...
def petersen_graph(create_using: Optional[Any] = ...): ...
def sedgewick_maze_graph(create_using: Optional[Any] = ...): ...
def tetrahedral_graph(create_using: Optional[Any] = ...): ...
def truncated_cube_graph(create_using: Optional[Any] = ...): ...
def truncated_tetrahedron_graph(create_using: Optional[Any] = ...): ...
def tutte_graph(create_using: Optional[Any] = ...): ...
