# Stubs for networkx.algorithms.flow.edmondskarp (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

from networkx.algorithms.flow.utils import *

def edmonds_karp(
    G,
    s,
    t,
    capacity: str = ...,
    residual: Optional[Any] = ...,
    value_only: bool = ...,
    cutoff: Optional[Any] = ...,
): ...
