# Stubs for networkx.algorithms.components.weakly_connected (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def weakly_connected_components(G): ...
def number_weakly_connected_components(G): ...
def weakly_connected_component_subgraphs(G, copy: bool = ...): ...
def is_weakly_connected(G): ...
