# Stubs for networkx.algorithms.assortativity.connectivity (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def average_degree_connectivity(
    G,
    source: str = ...,
    target: str = ...,
    nodes: Optional[Any] = ...,
    weight: Optional[Any] = ...,
): ...

k_nearest_neighbors = average_degree_connectivity
