# Stubs for networkx.algorithms.centrality.reaching (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def global_reaching_centrality(G, weight: Optional[Any] = ..., normalized: bool = ...): ...
def local_reaching_centrality(
    G,
    v,
    paths: Optional[Any] = ...,
    weight: Optional[Any] = ...,
    normalized: bool = ...,
): ...
