# Stubs for networkx.algorithms.approximation.steinertree (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def metric_closure(G, weight: str = ...): ...
def steiner_tree(G, terminal_nodes, weight: str = ...): ...
