# Stubs for networkx.algorithms.bipartite.centrality (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def degree_centrality(G, nodes): ...
def betweenness_centrality(G, nodes): ...
def closeness_centrality(G, nodes, normalized: bool = ...): ...
