# Stubs for networkx.algorithms.efficiency (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def efficiency(G, u, v): ...
def global_efficiency(G): ...
def local_efficiency(G): ...
