# Stubs for networkx.algorithms.reciprocity (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def reciprocity(G, nodes: Optional[Any] = ...): ...
def overall_reciprocity(G): ...
