# Stubs for networkx.linalg.modularitymatrix (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def modularity_matrix(G, nodelist: Optional[Any] = ..., weight: Optional[Any] = ...): ...
def directed_modularity_matrix(G, nodelist: Optional[Any] = ..., weight: Optional[Any] = ...): ...
