# Stubs for networkx.algorithms.tree.operations (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def join(rooted_trees, label_attribute: Optional[Any] = ...): ...
