# Stubs for networkx.algorithms.bipartite.cluster (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def latapy_clustering(G, nodes: Optional[Any] = ..., mode: str = ...): ...

clustering = latapy_clustering

def average_clustering(G, nodes: Optional[Any] = ..., mode: str = ...): ...
def robins_alexander_clustering(G): ...
