# Stubs for networkx.generators.intersection (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def uniform_random_intersection_graph(n, m, p, seed: Optional[Any] = ...): ...
def k_random_intersection_graph(n, m, k): ...
def general_random_intersection_graph(n, m, p): ...
