# Stubs for networkx.algorithms.components.connected (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def connected_components(G): ...
def connected_component_subgraphs(G, copy: bool = ...): ...
def number_connected_components(G): ...
def is_connected(G): ...
def node_connected_component(G, n): ...
