# Stubs for networkx.algorithms.simple_paths (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Optional

def is_simple_path(G, nodes): ...
def all_simple_paths(G, source, target, cutoff: Optional[Any] = ...): ...
def shortest_simple_paths(G, source, target, weight: Optional[Any] = ...): ...

class PathBuffer:
    paths: Any = ...
    sortedpaths: Any = ...
    counter: Any = ...
    def __init__(self) -> None: ...
    def __len__(self): ...
    def push(self, cost, path): ...
    def pop(self): ...
