# Stubs for networkx.algorithms.communicability_alg (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from networkx.utils import *

def communicability(G): ...
def communicability_exp(G): ...
