# Stubs for networkx.readwrite.leda (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def read_leda(path, encoding: str = ...): ...
def parse_leda(lines): ...
