# Stubs for networkx.algorithms.flow.networksimplex (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

range = xrange

def network_simplex(G, demand: str = ..., capacity: str = ..., weight: str = ...): ...
